package fr.titionfire.ffsaf.ws.send;

import fr.titionfire.ffsaf.data.model.CardModel;
import fr.titionfire.ffsaf.ws.CompetitionWS;
import fr.titionfire.ffsaf.ws.recv.RCard;
import io.quarkus.websockets.next.WebSocketConnection;

import java.util.List;

public class SSCard {

    public static void sendCards(WebSocketConnection connection, List<CardModel> cardModel) {
        CompetitionWS.sendNotifyToOtherEditor(connection, "sendCards", cardModel);
    }

    public static void sendRmCards(WebSocketConnection connection, List<Long> ids) {
        CompetitionWS.sendNotifyToOtherEditor(connection, "rmCards", ids);
    }

    public static void sendTeamCard(WebSocketConnection connection, RCard.SendTeamCards teamCards) {
        CompetitionWS.sendNotifyToOtherEditor(connection, "sendTeamCard", teamCards);
    }

    public static void rmTeamCard(WebSocketConnection connection, RCard.SendTeamCards teamCards) {
        CompetitionWS.sendNotifyToOtherEditor(connection, "rmTeamCard", teamCards);
    }
}
